//************************************************************//
//                                                            //
// Nom du produit : Triple Yahtzee                            //
// Nom du fichier : des.pas                                   //
// Auteur : Alcatiz                                           //
// Copyright : © LPJJl Software                               //
// Date/Heure : 23/08/2016 12:46:06                           //
// Version : 3.1                                              //
//                                                            //
//************************************************************//

{ Description : Jeu de Triple Yahtzee à plusieurs joueurs sur un même poste

Copyright (C) 2016   Alcatiz  lpjjl@skynet.be

This source is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your option)
any later version.

This code is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

A copy of the GNU General Public License is available on the World Wide Web
at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA.
}




unit des;

(* ---------- Gestion des dés ---------- *)

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, StdCtrls, ExtCtrls, Graphics;

const

  { Paramètres de Dessin }

  CArrondi = 16;   (* Taille de l'arrondi aux coins du dé *)
  CRayon = 6;      (* Rayon d'un point du dé *)
  CDelta = 17;     (* Distance de la zone de FIGURES au bord du dé *)

type

  { TDe }

  TDe = class
    strict private
      (* ----- Champs ----- *)
      FMontant : Integer;
      FGarder : Boolean;
      FImage : TImage;
      FEtat : TStaticText;
    private
      (* ----- Getters et setters ----- *)
      procedure SetGarder (AValue : Boolean);
      procedure SetMontant (AValue : Integer);
      (* ----- Méthodes ----- *)
      procedure Dessin_1 (AImage : TImage);
         (* Dessin d'un 1 *)
      procedure Dessin_2 (AImage : TImage);
         (* Dessin d'un 2 *)
      procedure Dessin_3 (AImage : TImage);
         (* Dessin d'un 3 *)
      procedure Dessin_4 (AImage : TImage);
         (* Dessin d'un 4 *)
      procedure Dessin_6 (AImage : TImage);
         (* Dessin d'un 6 *)
    public
      (* ----- Méthodes publiques ----- *)
      procedure Dessin;
         (* Dessin du dé *)
      procedure Lancer;
         (* Lancer du dé *)
      (* ----- Propriétés ----- *)
      property Montant : Integer read FMontant write SetMontant;
         (* Valeur du dé *)
      property Garder : Boolean read FGarder write SetGarder;
         (* Indique si le dé doit être conservé ou relancé *)
      property Image : TImage read FImage write FImage;
         (* Image du dé *)
      property Etat : TStaticText read FEtat write FEtat;
         (* Cadre associé affichaznt s'il faut garder ou relancer le dé *)
  end;

implementation

uses
  BGRABitmap, BGRABitmapTypes;

{ TDe }

procedure TDe.SetGarder(AValue: Boolean);
(* Setter de la propriété Garder - Modifie le libellé du bouton associé - Dessin du dé *)
begin
  if FGarder = AValue
     then
       Exit;
  FGarder := AValue;
  if FGarder
     then
       Etat.Caption := 'Garder'
     else
       Etat.Caption := 'Relancer';
  Dessin;
end;

procedure TDe.SetMontant(AValue: Integer);
(* Setter de la propriété Montant - Dessin du dé *)
begin
  if FMontant = AValue
     then
       Exit;
  FMontant := AValue;
  if FMontant in [1..6]
     then
       Dessin;
end;

procedure TDe.Dessin_1(AImage: TImage);
(* Dessin d'un 1 *)
begin
  with AImage.Canvas do
    Ellipse(Width div 2 - CRayon, Height div 2 - CRayon, Width div 2 + CRayon, Height div 2 + CRayon);
end;

procedure TDe.Dessin_2(AImage: TImage);
(* Dessin d'un 2 *)
begin
  with AImage.Canvas do
    begin
      Ellipse(CDelta - CRayon, CDelta - CRayon, CDelta + CRayon, CDelta + CRayon);
      Ellipse(Width - CDelta - CRayon, Height - CDelta - CRayon, Width - CDelta + CRayon, Height - CDelta + CRayon);
    end;
end;

procedure TDe.Dessin_3(AImage: TImage);
(* Dessin d'un 3 *)
begin
  Dessin_1(AImage);
  with AImage.Canvas do
    begin
      Ellipse(Width - CDelta - CRayon, CDelta - CRayon, Width - CDelta + CRayon, CDelta + CRayon);
      Ellipse(CDelta - CRayon, Height - CDelta - CRayon, CDelta + CRayon, Height - CDelta + CRayon);
    end;
end;

procedure TDe.Dessin_4(AImage: TImage);
(* Dessin d'un 4 *)
begin
  with AImage.Canvas do
    begin
      Ellipse(CDelta - CRayon, CDelta - CRayon, CDelta + CRayon, CDelta + CRayon);
      Ellipse(Width - CDelta - CRayon, CDelta - CRayon, Width - CDelta + CRayon, CDelta + CRayon);
      Ellipse(CDelta - CRayon, Height - CDelta - CRayon, CDelta + CRayon, Height - CDelta + CRayon);
      Ellipse(Width - CDelta - CRayon, Height - CDelta - CRayon, Width - CDelta + CRayon, Height - CDelta + CRayon);
    end;
end;

procedure TDe.Dessin_6(AImage: TImage);
(* Dessin d'un 6 *)
begin
  Dessin_4(AImage);
  with AImage.Canvas do
    begin
      Ellipse(CDelta - CRayon, Height div 2 - CRayon, CDelta + CRayon, Height div 2 + CRayon);
      Ellipse(Width - CDelta - CRayon, Height div 2 - CRayon, Width - CDelta + CRayon, Height div 2 + CRayon);
    end;
end;

procedure TDe.Dessin;
(* Dessin d'un dé *)
var
  bmp: TBGRABitmap;
  pxl: TBGRAPixel;

  procedure DessinPoints(const AMontant: integer);
  {$I constantes.inc}
  var
    i: integer;
  begin
    for i := 1 to 7 do
      if FIGURES[AMontant, i] = 1 then
        bmp.FillEllipseLinearColorAntialias(
          COORDONNEES[i, 1] * bmp.Width,
          COORDONNEES[i, 2] * bmp.Height,
          6,
          6,
          ColorToBGRA(clDkGray),
          //BGRA(255, 0, 0),
          BGRAPixelTransparent
        );
  end;

begin
  {
  with Image.Canvas do
    begin
      if Garder
         then
           Brush.Color := clMoneyGreen
         else
           Brush.Color := clCream;
      Pen.Color := clBlack;
      RoundRect(0, 0, Width, Height, CArrondi, CArrondi);
      Brush.Color := clDkGray;
    end;
  case Montant of
    1 : Dessin_1(Image);
    2 : Dessin_2(Image);
    3 : Dessin_3(Image);
    4 : Dessin_4(Image);
    5 : begin
          Dessin_3(Image);
          Dessin_2(Image);
        end;
    6 : Dessin_6(Image);
  end;
  }

  { http://forum.lazarus.freepascal.org/index.php/topic,32111.msg206707.html#msg206707 }
  Image.Picture.Bitmap.SetSize(Image.Width, Image.Height);

  bmp := TBGRABitmap.Create(
    Image.Picture.Bitmap.Width,
    Image.Picture.Bitmap.Height,
    BGRABlack
  );

  if Garder
     then
       pxl := ColorToBGRA(clMoneyGreen)
     else
       pxl := ColorToBGRA(clCream);

  (*
  bmp.FillRect(
    0,
    0,
    Image.Picture.Bitmap.Width,
    Image.Picture.Bitmap.Height,
    //BGRA(255, 192, 0),
    pxl,
    dmSet
  );
  *)

  bmp.FillRoundRectAntialias(
    0,
    0,
    Image.Picture.Bitmap.Width,
    Image.Picture.Bitmap.Height,
    cArrondi,
    cArrondi,
    pxl,
    []
  );

  DessinPoints(Montant);

  bmp.Draw(Image.Picture.Bitmap.Canvas, 0, 0, True);
  bmp.Free;

  Image.Invalidate;
end;

procedure TDe.Lancer;
(* Lancer du dé *)
begin
  Montant := Random(6) + 1;
end;

end.
